@@ -16,21 +16,19 @@ const Layout = imports.ui.layout;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
-const StatusMenu = imports.ui.statusMenu;
+const UserMenu = imports.ui.userMenu;
 const DateMenu = imports.ui.dateMenu;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
 const PANEL_ICON_SIZE = 24;
 
-const STARTUP_ANIMATION_TIME = 0.2;
-
 const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
 
 const ANIMATED_ICON_UPDATE_TIMEOUT = 100;
 const SPINNER_ANIMATION_TIME = 0.2;
 
-const STANDARD_TRAY_ICON_ORDER = ['a11y', 'display', 'keyboard', 'volume', 'bluetooth', 'network', 'battery'];
+const STANDARD_TRAY_ICON_ORDER = ['a11y', 'keyboard', 'volume', 'bluetooth', 'network', 'battery'];
 const STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION = {
     'a11y': imports.ui.status.accessibility.ATIndicator,
     'volume': imports.ui.status.volume.Indicator,
@@ -47,6 +45,14 @@ try {
     log('NMApplet is not supported. It is possible that your NetworkManager version is too old');
 }
 
+const GDM_TRAY_ICON_ORDER = ['a11y', 'display', 'keyboard', 'volume', 'battery'];
+const GDM_TRAY_ICON_SHELL_IMPLEMENTATION = {
+    'a11y': imports.ui.status.accessibility.ATIndicator,
+    'volume': imports.ui.status.volume.Indicator,
+    'battery': imports.ui.status.power.Indicator,
+    'keyboard': imports.ui.status.keyboard.XKBIndicator
+};
+
 // To make sure the panel corners blend nicely with the panel,
 // we draw background and borders the same way, e.g. drawing
 // them as filled shapes from the outside inwards instead of
@@ -237,7 +243,6 @@ AppMenuButton.prototype = {
 
     _init: function() {
         PanelMenu.Button.prototype._init.call(this, 0.0);
-        this._metaDisplay = global.screen.get_display();
         this._startingApps = [];
 
         this._targetApp = null;
@@ -698,12 +703,94 @@ function PanelCorner(panel, side) {
 }
 
 PanelCorner.prototype = {
-    _init: function(panel, side) {
-        this._panel = panel;
+    _init: function(box, side) {
         this._side = side;
+
+        this._box = box;
+        this._box.connect('style-changed', Lang.bind(this, this._boxStyleChanged));
+
         this.actor = new St.DrawingArea({ style_class: 'panel-corner' });
+        this.actor.connect('style-changed', Lang.bind(this, this._styleChanged));
         this.actor.connect('repaint', Lang.bind(this, this._repaint));
-        this.actor.connect('style-changed', Lang.bind(this, this.relayout));
+    },
+
+    _findRightmostButton: function(container) {
+        if (!container.get_children)
+            return null;
+
+        let children = container.get_children();
+
+        if (!children || children.length == 0)
+            return null;
+
+        // Start at the back and work backward
+        let index = children.length - 1;
+        while (!children[index].visible && index >= 0)
+            index--;
+
+        if (index < 0)
+            return null;
+
+        if (!(children[index].has_style_class_name('panel-menu')) &&
+            !(children[index].has_style_class_name('panel-button')))
+            return this._findRightmostButton(children[index]);
+
+        return children[index];
+    },
+
+    _findLeftmostButton: function(container) {
+        if (!container.get_children)
+            return null;
+
+        let children = container.get_children();
+
+        if (!children || children.length == 0)
+            return null;
+
+        // Start at the front and work forward
+        let index = 0;
+        while (!children[index].visible && index < children.length)
+            index++;
+
+        if (index == children.length)
+            return null;
+
+        if (!(children[index].has_style_class_name('panel-menu')) &&
+            !(children[index].has_style_class_name('panel-button')))
+            return this._findLeftmostButton(children[index]);
+
+        return children[index];
+    },
+
+    _boxStyleChanged: function() {
+        let button;
+
+        if (this._side == St.Side.LEFT)
+            button = this._findLeftmostButton(this._box);
+        else if (this._side == St.Side.RIGHT)
+            button = this._findRightmostButton(this._box);
+
+        if (button) {
+            if (this._button && this._buttonStyleChangedSignalId)
+                this._button.disconnect(this._buttonStyleChangedSignalId);
+
+            this._button = button;
+
+            button.connect('destroy', Lang.bind(this,
+                function() {
+                    if (this._button == button) {
+                        this._button = null;
+                        this._buttonStyleChangedSignalId = 0;
+                    }
+                }));
+
+            // Synchronize the locate button's pseudo classes with this corner
+            this._buttonStyleChangedSignalId = button.connect('style-changed', Lang.bind(this,
+                function(actor) {
+                    let pseudoClass = button.get_style_pseudo_class();
+                    this.actor.set_style_pseudo_class(pseudoClass);
+                }));
+        }
     },
 
     _repaint: function() {
@@ -766,20 +853,14 @@ PanelCorner.prototype = {
         cr.restore();
     },
 
-    relayout: function() {
+    _styleChanged: function() {
         let node = this.actor.get_theme_node();
 
         let cornerRadius = node.get_length("-panel-corner-radius");
         let innerBorderWidth = node.get_length('-panel-corner-inner-border-width');
 
-        this.actor.set_size(cornerRadius,
-                            innerBorderWidth + cornerRadius);
-        if (this._side == St.Side.LEFT)
-            this.actor.set_position(this._panel.actor.x,
-                                    this._panel.actor.y + this._panel.actor.height - innerBorderWidth);
-        else
-            this.actor.set_position(this._panel.actor.x + this._panel.actor.width - cornerRadius,
-                                    this._panel.actor.y + this._panel.actor.height - innerBorderWidth);
+        this.actor.set_size(cornerRadius, innerBorderWidth + cornerRadius);
+        this.actor.set_anchor_point(0, innerBorderWidth);
     }
 };
 
@@ -790,9 +871,8 @@ function Panel() {
 
 Panel.prototype = {
     _init : function() {
-        this.actor = new St.BoxLayout({ style_class: 'menu-bar',
-                                        name: 'panel',
-                                        reactive: true });
+        this.actor = new Shell.GenericContainer({ name: 'panel',
+                                                  reactive: true });
         this.actor._delegate = this;
 
         this._statusArea = {};
@@ -804,113 +884,60 @@ Panel.prototype = {
             this.actor.remove_style_class_name('in-overview');
         }));
 
-        this._leftPointerBarrier = 0;
-        this._rightPointerBarrier = 0;
+        if (global.session_type == Shell.SessionType.GDM) {
+            this._tray_icon_order = GDM_TRAY_ICON_ORDER;
+            this._tray_icon_shell_implementation = GDM_TRAY_ICON_SHELL_IMPLEMENTATION;
+        } else {
+            this._tray_icon_order = STANDARD_TRAY_ICON_ORDER;
+            this._tray_icon_shell_implementation = STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION;
+        }
+
         this._menus = new PopupMenu.PopupMenuManager(this);
 
         this._leftBox = new St.BoxLayout({ name: 'panelLeft' });
+        this.actor.add_actor(this._leftBox);
         this._centerBox = new St.BoxLayout({ name: 'panelCenter' });
+        this.actor.add_actor(this._centerBox);
         this._rightBox = new St.BoxLayout({ name: 'panelRight' });
+        this.actor.add_actor(this._rightBox);
 
-        this._leftCorner = new PanelCorner(this, St.Side.LEFT);
-        this._rightCorner = new PanelCorner(this, St.Side.RIGHT);
+        if (this.actor.get_direction() == St.TextDirection.RTL)
+            this._leftCorner = new PanelCorner(this._rightBox, St.Side.LEFT);
+        else
+            this._leftCorner = new PanelCorner(this._leftBox, St.Side.LEFT);
 
-        /* This box container ensures that the centerBox is positioned in the *absolute*
-         * center, but can be pushed aside if necessary. */
-        this._boxContainer = new Shell.GenericContainer();
-        this.actor.add(this._boxContainer, { expand: true });
-        this._boxContainer.add_actor(this._leftBox);
-        this._boxContainer.add_actor(this._centerBox);
-        this._boxContainer.add_actor(this._rightBox);
-        this._boxContainer.connect('get-preferred-width', Lang.bind(this, function(box, forHeight, alloc) {
-            let children = box.get_children();
-            for (let i = 0; i < children.length; i++) {
-                let [childMin, childNatural] = children[i].get_preferred_width(forHeight);
-                alloc.min_size += childMin;
-                alloc.natural_size += childNatural;
-            }
-        }));
-        this._boxContainer.connect('get-preferred-height', Lang.bind(this, function(box, forWidth, alloc) {
-            let children = box.get_children();
-            for (let i = 0; i < children.length; i++) {
-                let [childMin, childNatural] = children[i].get_preferred_height(forWidth);
-                if (childMin > alloc.min_size)
-                    alloc.min_size = childMin;
-                if (childNatural > alloc.natural_size)
-                    alloc.natural_size = childNatural;
-            }
-        }));
-        this._boxContainer.connect('allocate', Lang.bind(this, function(container, box, flags) {
-            let allocWidth = box.x2 - box.x1;
-            let allocHeight = box.y2 - box.y1;
-            let [leftMinWidth, leftNaturalWidth] = this._leftBox.get_preferred_width(-1);
-            let [centerMinWidth, centerNaturalWidth] = this._centerBox.get_preferred_width(-1);
-            let [rightMinWidth, rightNaturalWidth] = this._rightBox.get_preferred_width(-1);
+        this.actor.add_actor(this._leftCorner.actor);
 
-            let sideWidth, centerWidth;
-            centerWidth = centerNaturalWidth;
-            sideWidth = (allocWidth - centerWidth) / 2;
+        if (this.actor.get_direction() == St.TextDirection.RTL)
+            this._rightCorner = new PanelCorner(this._leftBox, St.Side.RIGHT);
+        else
+            this._rightCorner = new PanelCorner(this._rightBox, St.Side.RIGHT);
+        this.actor.add_actor(this._rightCorner.actor);
 
-            let childBox = new Clutter.ActorBox();
-
-            childBox.y1 = 0;
-            childBox.y2 = allocHeight;
-            if (this.actor.get_direction() == St.TextDirection.RTL) {
-                childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth),
-                                                    leftNaturalWidth);
-                childBox.x2 = allocWidth;
-            } else {
-                childBox.x1 = 0;
-                childBox.x2 = Math.min(Math.floor(sideWidth),
-                                       leftNaturalWidth);
-            }
-            this._leftBox.allocate(childBox, flags);
-
-            childBox.x1 = Math.ceil(sideWidth);
-            childBox.y1 = 0;
-            childBox.x2 = childBox.x1 + centerWidth;
-            childBox.y2 = allocHeight;
-            this._centerBox.allocate(childBox, flags);
-
-            childBox.y1 = 0;
-            childBox.y2 = allocHeight;
-            if (this.actor.get_direction() == St.TextDirection.RTL) {
-                childBox.x1 = 0;
-                childBox.x2 = Math.min(Math.floor(sideWidth),
-                                       rightNaturalWidth);
-            } else {
-                childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth),
-                                                    rightNaturalWidth);
-                childBox.x2 = allocWidth;
-            }
-            this._rightBox.allocate(childBox, flags);
-        }));
+        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        this.actor.connect('allocate', Lang.bind(this, this._allocate));
 
         /* Button on the left side of the panel. */
-        this._activitiesButton = new ActivitiesButton();
-        this._activities = this._activitiesButton.actor;
-        this._leftBox.add(this._activities);
+        if (global.session_type == Shell.SessionType.USER) {
+            this._activitiesButton = new ActivitiesButton();
+            this._activities = this._activitiesButton.actor;
+            this._leftBox.add(this._activities);
 
-        // The activities button has a pretend menu, so as to integrate
-        // more cleanly with the rest of the panel
-        this._menus.addMenu(this._activitiesButton.menu);
+            // The activities button has a pretend menu, so as to integrate
+            // more cleanly with the rest of the panel
+            this._menus.addMenu(this._activitiesButton.menu);
 
-        // Synchronize the button's pseudo classes with its corner
-        this._activities.connect('style-changed', Lang.bind(this,
-            function(actor) {
-                let rtl = actor.get_direction() == St.TextDirection.RTL;
-                let corner = rtl ? this._rightCorner : this._leftCorner;
-                let pseudoClass = actor.get_style_pseudo_class();
-                corner.actor.set_style_pseudo_class(pseudoClass);
-            }));
-
-        this._appMenu = new AppMenuButton();
-        this._leftBox.add(this._appMenu.actor);
-
-        this._menus.addMenu(this._appMenu.menu);
+            this._appMenu = new AppMenuButton();
+            this._leftBox.add(this._appMenu.actor);
+            this._menus.addMenu(this._appMenu.menu);
+        }
 
         /* center */
-        this._dateMenu = new DateMenu.DateMenuButton();
+        if (global.session_type == Shell.SessionType.USER)
+            this._dateMenu = new DateMenu.DateMenuButton({ showEvents: true });
+        else
+            this._dateMenu = new DateMenu.DateMenuButton({ showEvents: false });
         this._centerBox.add(this._dateMenu.actor, { y_fill: true });
         this._menus.addMenu(this._dateMenu.menu);
 
@@ -926,113 +953,145 @@ Panel.prototype = {
         this._rightBox.add(this._trayBox);
         this._rightBox.add(this._statusBox);
 
-        this._userMenu = new StatusMenu.StatusMenuButton();
-        this._userMenu.actor.name = 'panelStatus';
-        this._rightBox.add(this._userMenu.actor);
-
-        // Synchronize the buttons pseudo classes with its corner
-        this._userMenu.actor.connect('style-changed', Lang.bind(this,
-            function(actor) {
-                let rtl = actor.get_direction() == St.TextDirection.RTL;
-                let corner = rtl ? this._leftCorner : this._rightCorner;
-                let pseudoClass = actor.get_style_pseudo_class();
-                corner.actor.set_style_pseudo_class(pseudoClass);
-            }));
+        if (global.session_type == Shell.SessionType.USER) {
+            this._userMenu = new UserMenu.UserMenuButton();
+            this._userMenu.actor.name = 'panelStatus';
+            this._rightBox.add(this._userMenu.actor);
+        }
 
         Main.statusIconDispatcher.connect('status-icon-added', Lang.bind(this, this._onTrayIconAdded));
         Main.statusIconDispatcher.connect('status-icon-removed', Lang.bind(this, this._onTrayIconRemoved));
 
-        Main.chrome.addActor(this.actor);
-        Main.chrome.addActor(this._leftCorner.actor, { affectsStruts: false,
-                                                       affectsInputRegion: false });
-        Main.chrome.addActor(this._rightCorner.actor, { affectsStruts: false,
-                                                        affectsInputRegion: false });
-
+        Main.layoutManager.panelBox.add(this.actor);
         Main.ctrlAltTabManager.addGroup(this.actor, _("Top Bar"), 'start-here',
                                         { sortGroup: CtrlAltTab.SortGroup.TOP });
+    },
 
-        Main.layoutManager.connect('monitors-changed', Lang.bind(this, this._relayout));
-        this._relayout();
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        alloc.min_size = -1;
+        alloc.natural_size = Main.layoutManager.primaryMonitor.width;
+    },
+
+    _getPreferredHeight: function(actor, forWidth, alloc) {
+        // We don't need to implement this; it's forced by the CSS
+        alloc.min_size = -1;
+        alloc.natural_size = -1;
+    },
+
+    _allocate: function(actor, box, flags) {
+        let allocWidth = box.x2 - box.x1;
+        let allocHeight = box.y2 - box.y1;
+
+        let [leftMinWidth, leftNaturalWidth] = this._leftBox.get_preferred_width(-1);
+        let [centerMinWidth, centerNaturalWidth] = this._centerBox.get_preferred_width(-1);
+        let [rightMinWidth, rightNaturalWidth] = this._rightBox.get_preferred_width(-1);
+
+        let sideWidth, centerWidth;
+        centerWidth = centerNaturalWidth;
+        sideWidth = (allocWidth - centerWidth) / 2;
+
+        let childBox = new Clutter.ActorBox();
+
+        childBox.y1 = 0;
+        childBox.y2 = allocHeight;
+        if (this.actor.get_direction() == St.TextDirection.RTL) {
+            childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth),
+                                                leftNaturalWidth);
+            childBox.x2 = allocWidth;
+        } else {
+            childBox.x1 = 0;
+            childBox.x2 = Math.min(Math.floor(sideWidth),
+                                   leftNaturalWidth);
+        }
+        this._leftBox.allocate(childBox, flags);
+
+        childBox.x1 = Math.ceil(sideWidth);
+        childBox.y1 = 0;
+        childBox.x2 = childBox.x1 + centerWidth;
+        childBox.y2 = allocHeight;
+        this._centerBox.allocate(childBox, flags);
+
+        childBox.y1 = 0;
+        childBox.y2 = allocHeight;
+        if (this.actor.get_direction() == St.TextDirection.RTL) {
+            childBox.x1 = 0;
+            childBox.x2 = Math.min(Math.floor(sideWidth),
+                                   rightNaturalWidth);
+        } else {
+            childBox.x1 = allocWidth - Math.min(Math.floor(sideWidth),
+                                                rightNaturalWidth);
+            childBox.x2 = allocWidth;
+        }
+        this._rightBox.allocate(childBox, flags);
+
+        let [cornerMinWidth, cornerWidth] = this._leftCorner.actor.get_preferred_width(-1);
+        let [cornerMinHeight, cornerHeight] = this._leftCorner.actor.get_preferred_width(-1);
+        childBox.x1 = 0;
+        childBox.x2 = cornerWidth;
+        childBox.y1 = allocHeight;
+        childBox.y2 = allocHeight + cornerHeight;
+        this._leftCorner.actor.allocate(childBox, flags);
+
+        let [cornerMinWidth, cornerWidth] = this._rightCorner.actor.get_preferred_width(-1);
+        let [cornerMinHeight, cornerHeight] = this._rightCorner.actor.get_preferred_width(-1);
+        childBox.x1 = allocWidth - cornerWidth;
+        childBox.x2 = allocWidth;
+        childBox.y1 = allocHeight;
+        childBox.y2 = allocHeight + cornerHeight;
+        this._rightCorner.actor.allocate(childBox, flags);
     },
 
     startStatusArea: function() {
-        for (let i = 0; i < STANDARD_TRAY_ICON_ORDER.length; i++) {
-            let role = STANDARD_TRAY_ICON_ORDER[i];
-            let constructor = STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION[role];
+        for (let i = 0; i < this._tray_icon_order.length; i++) {
+            let role = this._tray_icon_order[i];
+            let constructor = this._tray_icon_shell_implementation[role];
             if (!constructor) {
                 // This icon is not implemented (this is a bug)
                 continue;
             }
-            let indicator = new constructor();
-            this._statusBox.add(indicator.actor);
-            this._menus.addMenu(indicator.menu);
 
-            this._statusArea[role] = indicator;
+            let indicator = new constructor();
+            this.addToStatusArea(role, indicator, i);
         }
 
         // PopupMenuManager depends on menus being added in order for
         // keyboard navigation
-        this._menus.addMenu(this._userMenu.menu);
+        if (this._userMenu)
+            this._menus.addMenu(this._userMenu.menu);
     },
 
-    startupAnimation: function() {
-        let oldY = this.actor.y;
-        this.actor.y = oldY - this.actor.height;
-        Tweener.addTween(this.actor,
-                         { y: oldY,
-                           time: STARTUP_ANIMATION_TIME,
-                           transition: 'easeOutQuad'
-                         });
+    addToStatusArea: function(role, indicator, position) {
+        if (this._statusArea[role])
+            throw new Error('Extension point conflict: there is already a status indicator for role ' + role);
 
-        let oldCornerY = this._leftCorner.actor.y;
-        this._leftCorner.actor.y = oldCornerY - this.actor.height;
-        this._rightCorner.actor.y = oldCornerY - this.actor.height;
-        Tweener.addTween(this._leftCorner.actor,
-                         { y: oldCornerY,
-                           time: STARTUP_ANIMATION_TIME,
-                           transition: 'easeOutQuad'
-                         });
-        Tweener.addTween(this._rightCorner.actor,
-                         { y: oldCornerY,
-                           time: STARTUP_ANIMATION_TIME,
-                           transition: 'easeOutQuad'
-                         });
-    },
+        if (!(indicator instanceof PanelMenu.Button))
+            throw new TypeError('Status indicator must be an instance of PanelMenu.Button');
 
-    _relayout: function() {
-        let primary = Main.layoutManager.primaryMonitor;
+        if (!position)
+            position = 0;
 
-        this.actor.set_position(primary.x, primary.y);
-        this.actor.set_size(primary.width, -1);
+        this._statusBox.insert_actor(indicator.actor, position);
+        this._menus.addMenu(indicator.menu);
 
-        if (this._leftPointerBarrier)
-            global.destroy_pointer_barrier(this._leftPointerBarrier);
-        if (this._rightPointerBarrier)
-            global.destroy_pointer_barrier(this._rightPointerBarrier);
+        this._statusArea[role] = indicator;
+        let destroyId = indicator.connect('destroy', Lang.bind(this, function(emitter) {
+            this._statusArea[role] = null;
+            emitter.disconnect(destroyId);
+        }));
 
-        this._leftPointerBarrier =
-            global.create_pointer_barrier(primary.x, primary.y,
-                                          primary.x, primary.y + this.actor.height,
-                                          1 /* BarrierPositiveX */);
-        this._rightPointerBarrier =
-            global.create_pointer_barrier(primary.x + primary.width, primary.y,
-                                          primary.x + primary.width, primary.y + this.actor.height,
-                                          4 /* BarrierNegativeX */);
-
-        this._leftCorner.relayout();
-        this._rightCorner.relayout();
+        return indicator;
     },
 
     _onTrayIconAdded: function(o, icon, role) {
         icon.height = PANEL_ICON_SIZE;
 
-        if (STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION[role]) {
+        if (this._tray_icon_shell_implementation[role]) {
             // This icon is legacy, and replaced by a Shell version
             // Hide it
             return;
         }
         // Figure out the index in our well-known order for this icon
-        let position = STANDARD_TRAY_ICON_ORDER.indexOf(role);
+        let position = this._tray_icon_order.indexOf(role);
         icon._rolePosition = position;
         let children = this._trayBox.get_children();
         let i;
