@@ -2,6 +2,7 @@
 
 const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
+const Signals = imports.signals;
 const St = imports.gi.St;
 
 const Lang = imports.lang;
@@ -23,10 +24,11 @@ Button.prototype = {
         this.actor._delegate = this;
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
         this.actor.connect('key-press-event', Lang.bind(this, this._onSourceKeyPress));
-        this.menu = new PopupMenu.PopupMenu(this.actor, menuAlignment, St.Side.TOP, 0);
+        this.menu = new PopupMenu.PopupMenu(this.actor, menuAlignment, St.Side.TOP);
+        this.menu.actor.add_style_class_name('panel-menu');
         this.menu.connect('open-state-changed', Lang.bind(this, this._onOpenStateChanged));
         this.menu.actor.connect('key-press-event', Lang.bind(this, this._onMenuKeyPress));
-        Main.chrome.addActor(this.menu.actor, { affectsStruts: false });
+        Main.uiGroup.add_actor(this.menu.actor);
         this.menu.actor.hide();
     },
 
@@ -79,8 +81,18 @@ Button.prototype = {
             this.actor.add_style_pseudo_class('active');
         else
             this.actor.remove_style_pseudo_class('active');
+    },
+
+    destroy: function() {
+        this.actor._delegate = null;
+
+        this.menu.destroy();
+        this.actor.destroy();
+
+        this.emit('destroy');
     }
 };
+Signals.addSignalMethods(Button.prototype);
 
 /* SystemStatusButton:
  *
