@@ -1,6 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const GConf = imports.gi.GConf;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -27,6 +28,8 @@ const CLOSE_BUTTON_FADE_TIME = 0.1;
 
 const DRAGGING_WINDOW_OPACITY = 100;
 
+const BUTTON_LAYOUT_KEY = '/desktop/gnome/shell/windows/button_layout';
+
 // Define a layout scheme for small window counts. For larger
 // counts we fall back to an algorithm. We need more schemes here
 // unless we have a really good algorithm.
@@ -91,22 +94,38 @@ function WindowClone(realWindow) {
 
 WindowClone.prototype = {
     _init : function(realWindow) {
-        this.actor = new Clutter.Clone({ source: realWindow.get_texture(),
-                                         reactive: true,
-                                         x: realWindow.x,
-                                         y: realWindow.y });
-        this.actor._delegate = this;
         this.realWindow = realWindow;
         this.metaWindow = realWindow.meta_window;
         this.metaWindow._delegate = this;
-        this.origX = realWindow.x;
-        this.origY = realWindow.y;
+
+        let [borderX, borderY] = this._getInvisibleBorderPadding();
+        this._windowClone = new Clutter.Clone({ source: realWindow.get_texture(),
+                                                x: -borderX,
+                                                y: -borderY });
+
+        this.origX = realWindow.x + borderX;
+        this.origY = realWindow.y + borderY;
+
+        let outerRect = realWindow.meta_window.get_outer_rect();
+
+        // The MetaShapedTexture that we clone has a size that includes
+        // the invisible border; this is inconvenient; rather than trying
+        // to compensate all over the place we insert a ClutterGroup into
+        // the hierarchy that is sized to only the visible portion.
+        this.actor = new Clutter.Group({ reactive: true,
+                                         x: this.origX,
+                                         y: this.origY,
+                                         width: outerRect.width,
+                                         height: outerRect.height });
+
+        this.actor.add_actor(this._windowClone);
+
+        this.actor._delegate = this;
 
         this._stackAbove = null;
 
-        this._sizeChangedId = this.realWindow.connect('size-changed', Lang.bind(this, function() {
-            this.emit('size-changed');
-        }));
+        this._sizeChangedId = this.realWindow.connect('size-changed',
+            Lang.bind(this, this._onRealWindowSizeChanged));
         this._realWindowDestroyId = this.realWindow.connect('destroy',
             Lang.bind(this, this._disconnectRealWindowSignals));
 
@@ -158,7 +177,7 @@ WindowClone.prototype = {
             // will look funny.
 
             if (!this._selected &&
-                this.metaWindow != global.screen.get_display().focus_window)
+                this.metaWindow != global.display.focus_window)
                 this._zoomEnd();
         }
     },
@@ -173,6 +192,32 @@ WindowClone.prototype = {
         this._realWindowDestroyId = 0;
     },
 
+    _getInvisibleBorderPadding: function() {
+        // We need to adjust the position of the actor because of the
+        // consequences of invisible borders -- in reality, the texture
+        // has an extra set of "padding" around it that we need to trim
+        // down.
+
+        // The outer rect paradoxically is the smaller rectangle,
+        // containing the positions of the visible frame. The input
+        // rect contains everything, including the invisible border
+        // padding.
+        let outerRect = this.metaWindow.get_outer_rect();
+        let inputRect = this.metaWindow.get_input_rect();
+        let [borderX, borderY] = [outerRect.x - inputRect.x,
+                                  outerRect.y - inputRect.y];
+
+        return [borderX, borderY];
+    },
+
+    _onRealWindowSizeChanged: function() {
+        let [borderX, borderY] = this._getInvisibleBorderPadding();
+        let outerRect = this.metaWindow.get_outer_rect();
+        this.actor.set_size(outerRect.width, outerRect.height);
+        this._windowClone.set_position(-borderX, -borderY);
+        this.emit('size-changed');
+    },
+
     _onDestroy: function() {
         this._disconnectRealWindowSignals();
 
@@ -222,8 +267,12 @@ WindowClone.prototype = {
         let [width, height] = this.actor.get_transformed_size();
 
         let monitorIndex = this.metaWindow.get_monitor();
-        let availArea = global.get_monitors()[monitorIndex];
-        if (monitorIndex == global.get_primary_monitor_index()) {
+        let monitor = Main.layoutManager.monitors[monitorIndex];
+        let availArea = new Meta.Rectangle({ x: monitor.x,
+                                             y: monitor.y,
+                                             width: monitor.width,
+                                             height: monitor.height });
+        if (monitorIndex == Main.layoutManager.primaryIndex) {
             availArea.y += Main.panel.actor.height;
             availArea.height -= Main.panel.actor.height;
         }
@@ -305,6 +354,21 @@ WindowClone.prototype = {
         this.emit('drag-begin');
     },
 
+    _getWorkspaceActor : function() {
+        let index = this.metaWindow.get_workspace().index();
+        return Main.overview.workspaces.getWorkspaceByIndex(index);
+    },
+
+    handleDragOver : function(source, actor, x, y, time) {
+        let workspace = this._getWorkspaceActor();
+        return workspace.handleDragOver(source, actor, x, y, time);
+    },
+
+    acceptDrop : function(source, actor, x, y, time) {
+        let workspace = this._getWorkspaceActor();
+        workspace.acceptDrop(source, actor, x, y, time);
+    },
+
     _onDragCancelled : function (draggable, time) {
         this.emit('drag-cancelled');
     },
@@ -397,6 +461,8 @@ WindowOverlay.prototype = {
 
     show: function() {
         this._hidden = false;
+        if (this._windowClone.actor.has_pointer)
+            this.closeButton.show();
         this.title.show();
     },
 
@@ -433,9 +499,20 @@ WindowOverlay.prototype = {
         let button = this.closeButton;
         let title = this.title;
 
+        let gconf = GConf.Client.get_default();
+        let layout = gconf.get_string(BUTTON_LAYOUT_KEY);
+        let rtl = St.Widget.get_default_direction() == St.TextDirection.RTL;
+
+        let split = layout.split(":");
+        let side;
+        if (split[0].indexOf("close") > -1)
+            side = rtl ? St.Side.RIGHT : St.Side.LEFT;
+        else
+            side = rtl ? St.Side.LEFT : St.Side.RIGHT;
+
         let buttonX;
         let buttonY = cloneY - (button.height - button._overlap);
-        if (St.Widget.get_default_direction() == St.TextDirection.RTL)
+        if (side == St.Side.LEFT)
             buttonX = cloneX - (button.width - button._overlap);
         else
             buttonX = cloneX + (cloneWidth - button._overlap);
@@ -562,7 +639,7 @@ Workspace.prototype = {
         this._height = 0;
 
         this.monitorIndex = monitorIndex;
-        this._monitor = global.get_monitors()[this.monitorIndex];
+        this._monitor = Main.layoutManager.monitors[this.monitorIndex];
         this._windowOverlaysGroup = new Clutter.Group();
         // Without this the drop area will be overlapped.
         this._windowOverlaysGroup.set_size(0, 0);
@@ -617,6 +694,7 @@ Workspace.prototype = {
             function () {
                 this._dropRect.set_position(x, y);
                 this._dropRect.set_size(width, height);
+                this.positionWindows(WindowPositionFlags.ANIMATE);
                 return false;
             }));
 
@@ -639,16 +717,6 @@ Workspace.prototype = {
         return this._windows.length == 0;
     },
 
-    /**
-     * setReactive:
-     * @reactive: %true iff the workspace should be reactive
-     *
-     * Set the workspace (desktop) reactive
-     **/
-    setReactive: function(reactive) {
-        this.actor.reactive = reactive;
-    },
-
     // Only use this for n <= 20 say
     _factorial: function(n) {
         let result = 1;
@@ -1414,7 +1482,7 @@ Workspace.prototype = {
                                                  time);
             return true;
         } else if (source.shellWorkspaceLaunch) {
-            source.shellWorkspaceLaunch({ workspace: this.metaWorkspace,
+            source.shellWorkspaceLaunch({ workspace: this.metaWorkspace ? this.metaWorkspace.index() : -1,
                                           timestamp: time });
             return true;
         }
